Package: glyparse
Title: Parsing Glycan Structure Text Representations
Version: 0.7.0
Authors@R: 
    person("Bin", "Fu", , "23110220018@m.fudan.edu.cn", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0001-8567-2997"))
Description: 
    Provides functions to parse glycan structure text representations
    into 'glyrepr' glycan structures. Currently, it supports StrucGP-style,
    pGlyco-style, IUPAC-condensed, IUPAC-extended, IUPAC-short, IUPAC-compact,
    WURCS, LINUCS, Linear Code, GlycoCT, and KCF format. It also provides an automatic parser
    to detect the format and parse the structure string.
License: MIT + file LICENSE
Suggests: knitr, rmarkdown, glue, testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
URL: https://glycoverse.github.io/glyparse/,
        https://github.com/glycoverse/glyparse
Imports: checkmate, cli, dplyr, glyrepr (>= 0.12.0), igraph, purrr (>=
        1.0.0), rlang, rstackdeque, stringr, vctrs
Depends: R (>= 4.1)
VignetteBuilder: knitr
BugReports: https://github.com/glycoverse/glyparse/issues
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-07-06 03:13:45 UTC; fubin
Author: Bin Fu [aut, cre, cph] (ORCID: <https://orcid.org/0000-0001-8567-2997>)
Maintainer: Bin Fu <23110220018@m.fudan.edu.cn>
Repository: CRAN
Date/Publication: 2026-07-06 06:10:14 UTC
Built: R 4.5.2; ; 2026-07-06 10:33:43 UTC; unix
