# parse_iupac_short works for a complex example

    Code
      parse_iupac_short(x)
    Output
      <glycan_structure[1]>
      [1] Neu5Ac(a2-3)Gal(b1-4)GlcNAc(b1-2)Man(a1-3)[Neu5Ac(a2-3)Gal(b1-4)GlcNAc(b1-2)Man(a1-6)]Man(b1-4)GlcNAc(b1-4)[Fuc(a1-6)]GlcNAc(b1-
      # Unique structures: 1

