# A2B2C1D1E2F1G3gfF5fedD1E2F1feE2F1fedcba

    Code
      print(glycan, verbose = TRUE)
    Output
      <glycan_structure[1]>
      [1] NeuAc(??-?)Hex(??-?)[dHex(??-?)]HexNAc(??-?)Hex(??-?)[Hex(??-?)HexNAc(??-?)[Hex(??-?)HexNAc(??-?)]Hex(??-?)]Hex(??-?)HexNAc(??-?)HexNAc(??-
      # Unique structures: 1

# A2B2C1D1E1F1fedD1E1F1feE1F1fedcba

    Code
      print(glycan, verbose = TRUE)
    Output
      <glycan_structure[1]>
      [1] Hex(??-?)Hex(??-?)Hex(??-?)[Hex(??-?)Hex(??-?)[Hex(??-?)Hex(??-?)]Hex(??-?)]Hex(??-?)HexNAc(??-?)HexNAc(??-
      # Unique structures: 1

