# glycan with only one monosacharide

    Code
      parse_wurcs(wurcs)
    Output
      <glycan_structure[1]>
      [1] GlcNAc(b1-
      # Unique structures: 1

# N-glycan core

    Code
      parse_wurcs(wurcs)
    Output
      <glycan_structure[1]>
      [1] Man(a1-3)[Man(a1-6)]Man(b1-4)GlcNAc(b1-4)GlcNAc(?1-
      # Unique structures: 1

# H5H4F1S2

    Code
      parse_wurcs(wurcs)
    Output
      <glycan_structure[1]>
      [1] Neu5Ac(a2-3)Gal(b1-4)GlcNAc(b1-2)Man(a1-3)[Neu5Ac(a2-3)Gal(b1-4)GlcNAc(b1-2)Man(a1-6)]Man(b1-4)GlcNAc(b1-4)[Fuc(a1-6)]GlcNAc(b1-
      # Unique structures: 1

