# (N(F)(N(H(H(N))(H(N(H))))))

    Code
      print(glycan, verbose = TRUE)
    Output
      <glycan_structure[1]>
      [1] Hex(??-?)HexNAc(??-?)Hex(??-?)[HexNAc(??-?)Hex(??-?)]Hex(??-?)HexNAc(??-?)[dHex(??-?)]HexNAc(??-
      # Unique structures: 1

# (N(F)(N(H(H(N(H)))(H(N(H(A)))))))

    Code
      print(glycan, verbose = TRUE)
    Output
      <glycan_structure[1]>
      [1] NeuAc(??-?)Hex(??-?)HexNAc(??-?)Hex(??-?)[Hex(??-?)HexNAc(??-?)Hex(??-?)]Hex(??-?)HexNAc(??-?)[dHex(??-?)]HexNAc(??-
      # Unique structures: 1

# single monosaccharide

    Code
      print(glycan, verbose = TRUE)
    Output
      <glycan_structure[1]>
      [1] HexNAc(??-
      # Unique structures: 1

# pH and aH monosaccharides

    Code
      print(glycan, verbose = TRUE)
    Output
      <glycan_structure[1]>
      [1] HexN(??-?)Hex?P(??-
      # Unique structures: 1

