# IUPAC-condensed: some O-glycan

    Code
      print(glycan, verbose = TRUE)
    Output
      <glycan_structure[1]>
      [1] Neu5Ac(a2-3)Gal(b1-4)[Fuc(a1-3)]GlcNAc(b1-6)[Neu5Ac(a2-3)Gal(b1-3)]GalNAc(b1-
      # Unique structures: 1

