% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse-linear-code.R
\name{parse_linear_code}
\alias{parse_linear_code}
\title{Parse Linear Code Structures}
\usage{
parse_linear_code(x, on_failure = "error")
}
\arguments{
\item{x}{A character vector of Linear Code strings. NA values are allowed and will be returned as NA structures.}

\item{on_failure}{How to handle parsing failures. \code{"error"} aborts when a
structure cannot be parsed. \code{"na"} returns \code{NA} at invalid positions.}
}
\value{
A \code{\link[glyrepr:glycan_structure]{glyrepr::glycan_structure()}} object.
}
\description{
Parse Linear Code structures into a \code{\link[glyrepr:glycan_structure]{glyrepr::glycan_structure()}}.
To know more about Linear Code, see \href{https://www.jstage.jst.go.jp/article/tigg1989/14/77/14_77_127/_article}{this article}.
}
\examples{
linear_code <- "Ma3(Ma6)Mb4GNb4GNb"
parse_linear_code(linear_code)

}
