% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse-pglyco.R
\name{parse_pglyco_struc}
\alias{parse_pglyco_struc}
\title{Parse pGlyco Structures}
\usage{
parse_pglyco_struc(x, on_failure = "error")
}
\arguments{
\item{x}{A character vector of pGlyco-style structure strings. NA values are allowed and will be returned as NA structures.}

\item{on_failure}{How to handle parsing failures. \code{"error"} aborts when a
structure cannot be parsed. \code{"na"} returns \code{NA} at invalid positions.}
}
\value{
A \code{\link[glyrepr:glycan_structure]{glyrepr::glycan_structure()}} object.
}
\description{
Parse pGlyco-style structure characters into a \code{\link[glyrepr:glycan_structure]{glyrepr::glycan_structure()}}.
See example below for the structure format.
}
\examples{
glycan <- parse_pglyco_struc("(N(F)(N(H(H(N))(H(N(H))))))")
print(glycan, verbose = TRUE)

}
